==> xinvest-2.5.1.tar.gz: xinvest-2.5.1/util.c <==
/*
  Xinvest is copyright 1995,1996 by Mark Buser.

  Permission is hereby granted to copy
  and freely distribute copies of this
  program for non-commercial purposes
  without fee, provided that this notice
  appears in all copies.
  
  All redistributions must be in their
  entirety as originally distributed.
  Feel free to modify Xinvest, but
  modified versions may not be distributed
  without prior consent of the author.
  
  This software is provided 'as-is'
  without any express or implied warranty.
  In no event will the author be held
  liable for any damages resulting from
  the use of this software.

  $Revision: 1.7 $ $Date: 1997/04/13 12:49:45 $
*/
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <Xm/Xm.h>

#include "account.h"
#include "file.h"
#include "status.h"
#include "util.h"

void syntax(int argc, char **argv)
{
  if (argc != 1)
    fprintf( stderr, "%s: unknown option \"%s\"\n", argv[0], argv[1]);
  fprintf( stderr, "usage: %s [options ...]\n", argv[0]);
  fprintf( stderr, "       where options include:\n\n");
  fprintf( stderr, "       -f filename(s)  to load transaction file(s),\n");
  fprintf( stderr, "       -help           to print this message,\n");
  fprintf( stderr, "       -install        to install a private colormap,\n");
  fprintf( stderr, "       or any valid X toolkit option.\n\n");
}


/* All I want is the lousy max path length, why is this a good thing? */
#ifdef PATH_MAX
static int pathmax = PATH_MAX;
#else
static int pathmax = 0;
#endif

#define PATH_MAX_GUESS 1024

int path_length ()
{
   if (pathmax == 0) {
     if ((pathmax = pathconf ("/", _PC_PATH_MAX)) < 0) 
       pathmax = PATH_MAX_GUESS;
     else
       pathmax++;
   }

   return (pathmax);
}


/* Look at an argc/argv pair for the -f filename[s] option.  Return argc/argv
** with this option removed.
*/
void removeFileOption ( int *argc, char **argv )
{
  int i, j;
  int start = -1;
  int end = -1;

  for (i = 0; i < *argc; i++) {
    /*
    ** Find -f option and the next option after it, if any.  No filenames
    ** are allowed to start with '-'.
    */
    if ( argv[i] && strcmp ( "-f", argv[i] ) == 0) {
      for (start = i, j = i+1; j < *argc; j++)
        if ( argv[j] && strncmp ("-", argv[j], strlen("-")) == 0) {
          end = j;
          break;
        }
    }

    if ( start > 0 ) {          /* Found -f option. */

      if ( end > 0 ) {          /* Option(s) after it, move up to discard -f. */
        int newargc = *argc - (end - start);

        for ( j = end; j < *argc; j++)
          argv[start++] = argv[j];

        if (end < *argc-1) {    /* There may be more -f's, look again */
          *argc = newargc;
          removeFileOption (argc, argv);
        } else {
          *argc = newargc;
        }
        break;

      } else if ( end == -1 ) { /* No options after it, just decrement argc */
        *argc = start;
        break;
      }
    }

    /* else no -f found, no changes required */
  }

  return;
}

/* Look at an argc/argv pair for -f filename[s] options. Process the option
** by loading and processing the files.
*/
void processFileOption ( int argc, char **argv)
{
  int account = 0;
  /*
  ** If any options left, they must be data files beginning with -f.
  ** There may be more than one -f option and each can have zero or more
  ** filenames [-f filename(s) -f filename(s) ...].
  */
  argv++; argc--;                          /* skip executable name */
  while (argc) {

    if ( strcmp ( "-f", *argv) != 0 ) {    /* bad option */
      char errmsg[132];
      sprintf ( errmsg,
                "Unknown option '%s' ignored,\nexpected '-f filename(s)'.",
                *argv );
      write_status ( errmsg, ERR);
      argv++; argc--;                      /* skip -f */

    } else {                               /* -f option, as expected */
      argv++; argc--; 

      while ( argc ) {                     /* any number of filenames */
        if (strncmp ( "-", *argv, strlen("-")) == 0)
          break;                           /* found option, go back to top */


        if ( (account = loadfile( *argv )) )
          processAccount( account );

        argv++; argc--;
      }

    }

  }
  if (account)
    displayAccount( account );
}


/*
** Host Specific Support
*/

/* Supplied by Cube-X */
#ifdef NeXT

extern char *getwd (char *pathname);

char *getcwd ( char *buf, size_t size)
{
  char realBuffer[MAXPATHLEN+1];
  int length;

  if (getwd(realBuffer) == NULL) {
    errno = EACCES;
    return NULL;
  }
  length = strlen(realBuffer);
  if (length >= size) {
    errno = ERANGE;
    return NULL;
  }
  strcpy (buf, realBuffer);
  return buf;
}

char *strdup ( const char *src)
{
  char *copy = (char *) malloc(strlen(src)+1);
  if (copy)
    strcpy(copy, src);
  return copy;
}
#endif


/* Unixware needs this. Supplied by Thanh Ma. */
#ifdef NEED_STRCASECMP

int
strncasecmp(char *s1, char *s2,int n)
{
    register int c1, c2,l=0;

    while (*s1 && *s2 && l<n) {
    c1 = tolower(*s1);
    c2 = tolower(*s2);
    if (c1 != c2)
        return (c1 - c2);
    s1++;
    s2++;
    l++;
    }
    return (int) (0);
}

int
strcasecmp(char *s1, char *s2)
{
    register int c1, c2;

    while (*s1 && *s2) {
    c1 = tolower(*s1);
    c2 = tolower(*s2);
    if (c1 != c2)
        return (c1 - c2);
    s1++;
    s2++;
    }
    return (int) (*s1 - *s2);
}
#endif


#if defined(__FreeBSD__) || defined (NEED_STRPTIME)

/* Weird, FreeBSD and IRIX 5 don't have this, but do have strftime. 
** This is a crippled implementation, only supporting the formats
** actually used in the rest of Xinvest.  Note that the return _value_
** does not conform to the man page either.
*/
char *strptime ( char* buf, char *fmt, struct tm *tm )
{
  char err_msg[128];

  if ( strcmp( fmt, "%m/%d/%y" ) == 0 ) 
    sscanf ( buf, "%d/%d/%d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );

  else if (strcmp ( fmt, "%m/%d/%Y" ) == 0 ) {
    sscanf ( buf, "%d/%d/%d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );
    tm->tm_year -= 1900;

  } else if (strcmp ( fmt, "%m %d %y" ) == 0 ) 
    sscanf ( buf, "%d %d %d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );

  else {
    sprintf ( err_msg, "Xinvest internal error: strptime.  Unsupported\n"
                      "format: \"%s\".  See Reporting Bugs.", fmt);
    write_status( err_msg, ERR);
  }

  tm->tm_mon--;  /* Months are from 0-11, mdays from 1-31, years since 1900 */

  return NULL;  /* completely bogus return value, but I don't use it */
}
#endif

==> xinvest-2.6.tar.gz: xinvest-2.6/util.c <==
/*
  Xinvest Copyright (c)1995-98 Mark Buser,
  All Rights Reserved.

  Permission is hereby granted to copy
  and freely distribute copies of this
  program for non-commercial purposes
  without fee, provided that this notice
  appears in all copies.
  
  All redistributions must be in their
  entirety as originally distributed.
  Feel free to modify Xinvest, but
  modified versions may not be distributed
  without prior consent of the author.
  
  This software is provided 'as-is'
  without any express or implied warranty.
  In no event will the author be held
  liable for any damages resulting from
  the use of this software.

  $Revision: 1.14 $ $Date: 1998/05/02 15:58:38 $
*/
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <Xm/Xm.h>

#include "account.h"
#include "calendar.h"
#include "file.h"
#ifdef STRFMON
#include "monetary.h"
#endif
#include "pref.h"
#include "status.h"
#include "util.h"

void syntax(int argc, char **argv)
{
  if (argc != 1)
    fprintf( stderr, "%s: unknown option \"%s\"\n", argv[0], argv[1]);
  fprintf( stderr, "usage: %s [options ...]\n", argv[0]);
  fprintf( stderr, "       where options include:\n\n");
  fprintf( stderr, "       -f filename(s)  to load transaction file(s),\n");
  fprintf( stderr, "       -help           to print this message,\n");
  fprintf( stderr, "       -install        to install a private colormap,\n");
  fprintf( stderr, "       or any valid X toolkit option.\n\n");
}


/* All I want is the lousy max path length, why is this a good thing? */
#ifdef PATH_MAX
static int pathmax = PATH_MAX;
#else
static int pathmax = 0;
#endif

#define PATH_MAX_GUESS 1024

int path_length ()
{
   if (pathmax == 0) {
     if ((pathmax = pathconf ("/", _PC_PATH_MAX)) < 0) 
       pathmax = PATH_MAX_GUESS;
     else
       pathmax++;
   }

   return (pathmax);
}


/* Look at an argc/argv pair for the -f filename[s] option.  Return argc/argv
** with this option removed.
*/
void removeFileOption ( int *argc, char **argv )
{
  int i, j;
  int start = -1;
  int end = -1;

  for (i = 0; i < *argc; i++) {
    /*
    ** Find -f option and the next option after it, if any.  No filenames
    ** are allowed to start with '-'.
    */
    if ( argv[i] && strcmp ( "-f", argv[i] ) == 0) {
      for (start = i, j = i+1; j < *argc; j++)
        if ( argv[j] && strncmp ("-", argv[j], strlen("-")) == 0) {
          end = j;
          break;
        }
    }

    if ( start > 0 ) {          /* Found -f option. */

      if ( end > 0 ) {          /* Option(s) after it, move up to discard -f. */
        int newargc = *argc - (end - start);

        for ( j = end; j < *argc; j++)
          argv[start++] = argv[j];

        if (end < *argc-1) {    /* There may be more -f's, look again */
          *argc = newargc;
          removeFileOption (argc, argv);
        } else {
          *argc = newargc;
        }
        break;

      } else if ( end == -1 ) { /* No options after it, just decrement argc */
        *argc = start;
        break;
      }
    }

    /* else no -f found, no changes required */
  }

  return;
}

/* Look at an argc/argv pair for -f filename[s] options. Process the option
** by loading and processing the files.
*/
void processFileOption ( int argc, char **argv)
{
  int account = 0;
  /*
  ** If any options left, they must be data files beginning with -f.
  ** There may be more than one -f option and each can have zero or more
  ** filenames [-f filename(s) -f filename(s) ...].
  */
  argv++; argc--;                          /* skip executable name */
  while (argc) {

    if ( strcmp ( "-f", *argv) != 0 ) {    /* bad option */
      sprintf ( errmsg,
                "Unknown option '%s' ignored,\nexpected '-f filename(s)'.",
                *argv );
      write_status ( errmsg, ERR);
      argv++; argc--;                      /* skip -f */

    } else {                               /* -f option, as expected */
      argv++; argc--; 

      while ( argc ) {                     /* any number of filenames */
        if (strncmp ( "-", *argv, strlen("-")) == 0)
          break;                           /* found option, go back to top */


        if ( (account = loadfile( *argv )) )
          processAccount( account );

        argv++; argc--;
      }

    }

  }
  if (account)
    displayAccount( account );
}

/* Format date from long format into preference format */
char *utilDateOut (long dateIn)
{
  static char dateStr[20];
  char *date = calLtoStr (dateIn);  /* in mm/dd/yyyy format */
  struct tm dateTm;

  strptime (date, "%m/%d/%Y", &dateTm);

  switch (prefGetDate()) {
    case PREF_DATE_LOC:
	 strftime (dateStr, 20, "%x", &dateTm);
         break;
    case PREF_DATE_DMY:
	 strftime (dateStr, 20, "%d/%m/%Y", &dateTm);
         break;
    default:
    case PREF_DATE_MDY:
	 strftime (dateStr, 20, "%m/%d/%Y", &dateTm);
         break;
  }
  return (dateStr);
}

/* Format monetary double into preference format */
char *utilCurrOut (double currIn, int currency)
{
  static char strval[40], tmp[40];
  char *from, *to;
  char groupc, decc;
  int leadd, i;
  int precision = prefGetPrec();

  switch (prefGetCurr()) {

#ifdef STRFMON
    case PREF_CURR_LOC:
                       if (currency==UTILCURR)
                         strfmon (strval, 40, "%.2n", currIn);
                       else {
			 char format[10];
			 sprintf (format, "%%!.%dn", precision);
                         strfmon (strval, 40, format, currIn);
		       }
                       return (strval);
#endif         

    case PREF_CURR_DC: 
		       groupc = '.';
                       decc = ',';
                       break;

    case PREF_CURR_CD:
    default:
		       groupc = ',';
                       decc = '.';
                       break;
  }

  sprintf (tmp, "%.*f", (currency==UTILCURR)?2:precision, currIn);

  /* find leading digits */
  leadd = strspn (tmp, "0123456789");
  to = strval;
  from = tmp;
    
  for (i=0; i < strlen(tmp) && i < 39; i++) {
    if (i==0 && currency==UTILCURR) {
      strcpy (to, prefGetCurrChar());
      to += strlen (prefGetCurrChar());
    }
    if (leadd) 
      if (leadd--%3 == 0 && i != 0) 
        *to++ = groupc;
    if (*from == '.' || *from == '.') {
      *to++ = decc;
      from++;
    } else
      *to++ = *from++;
  }
  *to = '\0';
                       
  return (strval);
}

/*
** Host Specific Support
*/

/* Supplied by Cube-X */
#ifdef NeXT

extern char *getwd (char *pathname);

char *getcwd ( char *buf, size_t size)
{
  char realBuffer[MAXPATHLEN+1];
  int length;

  if (getwd(realBuffer) == NULL) {
    errno = EACCES;
    return NULL;
  }
  length = strlen(realBuffer);
  if (length >= size) {
    errno = ERANGE;
    return NULL;
  }
  strcpy (buf, realBuffer);
  return buf;
}

char *strdup ( const char *src)
{
  char *copy = (char *) malloc(strlen(src)+1);
  if (copy)
    strcpy(copy, src);
  return copy;
}
#endif


/* Unixware needs this. Supplied by Thanh Ma. */
#ifdef NEED_STRCASECMP

int
strncasecmp(char *s1, char *s2,int n)
{
    register int c1, c2,l=0;

    while (*s1 && *s2 && l<n) {
    c1 = tolower(*s1);
    c2 = tolower(*s2);
    if (c1 != c2)
        return (c1 - c2);
    s1++;
    s2++;
    l++;
    }
    return (int) (0);
}

int
strcasecmp(char *s1, char *s2)
{
    register int c1, c2;

    while (*s1 && *s2) {
    c1 = tolower(*s1);
    c2 = tolower(*s2);
    if (c1 != c2)
        return (c1 - c2);
    s1++;
    s2++;
    }
    return (int) (*s1 - *s2);
}
#endif


#if defined (NEED_STRPTIME)

/* Weird, FreeBSD (< 2.2?) and IRIX 5.x don't have this, but do have strftime. 
** This is a crippled implementation, only supporting the formats
** actually used in the rest of Xinvest.  Note that the return _value_
** does not conform to the man page either.
*/
char *strptime ( char* buf, char *fmt, struct tm *tm )
{

  if ( strcmp( fmt, "%m/%d/%y" ) == 0 ) 
    sscanf ( buf, "%d/%d/%d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );

  else if (strcmp ( fmt, "%m/%d/%Y" ) == 0 ) {
    sscanf ( buf, "%d/%d/%d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );
    tm->tm_year -= 1900;

  } else if (strcmp ( fmt, "%m %d %y" ) == 0 ) 
    sscanf ( buf, "%d %d %d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );

  else {
    sprintf ( errmsg, "Xinvest internal error: strptime.  Unsupported\n"
                      "format: \"%s\".  See Reporting Bugs.", fmt);
    write_status( errmsg, ERR);
  }

  tm->tm_mon--;  /* Months are from 0-11, mdays from 1-31, years since 1900 */

  return NULL;  /* completely bogus return value, but I don't use it */
}
#endif
